@@ -1,5 +1,5 @@
 /* insertion.c -- insertions for Texinfo.
-   $Id: insertion.c,v 1.36 2004/03/19 16:39:43 karl Exp $
+   $Id: insertion.c,v 1.38 2004/04/06 21:27:32 dirt Exp $
 
    Copyright (C) 1998, 1999, 2000, 2001, 2002, 2003, 2004 Free Software
    Foundation, Inc.
@@ -1117,18 +1117,18 @@ end_insertion (int type)
       close_insertion_paragraph ();
       current_indent -= default_indentation_increment;
       if (html)
-        add_word ("</ol>\n");
+        add_html_block_elt ("</ol>\n");
       break;
 
     case flushleft:
       if (html)
-        add_word ("</div>\n");
+        add_html_block_elt ("</div>\n");
       close_insertion_paragraph ();
       break;
 
     case cartouche:
       if (html)
-	add_word ("</td></tr></table>\n");
+	add_html_block_elt ("</td></tr></table>\n");
       close_insertion_paragraph ();
       break;
 
@@ -1220,6 +1220,10 @@ end_insertion (int type)
          start of a new paragraph, except for the XML output. */
       if (!xml || docbook)
         close_insertion_paragraph ();
+
+      /* </pre> closes paragraph without messing with </p>.  */
+      if (html && type != quotation)
+          paragraph_is_open = 0;
       break;
 
     case table:
@@ -1966,19 +1970,24 @@ cm_author (void)
   if (is_in_insertion_of_type (quotation))
     {
       if (html)
+        add_word_args ("&mdash; %s", rest);
+      else if (docbook)
         {
-          add_word ("&mdash;");
+          /* FIXME Ideally, we should use an attribution element,
+             but they are supposed to be at the start of quotation
+             blocks.  So to avoid looking ahead mess, let's just
+             use mdash like HTML for now.  */
+          xml_insert_entity ("mdash");
+          add_word (rest);
         }
       else if (xml)
         {
-          xml_insert_entity ("mdash");
+          xml_insert_element (AUTHOR, START);
+          add_word (rest);
+          xml_insert_element (AUTHOR, END);
         }
       else
-        {
-          add_word ("--");
-        }
-
-      add_word (rest);
+        add_word_args ("-- %s", rest);
     }
   else if (is_in_insertion_of_type (titlepage))
     {
